//----------------------------------------------------------------------
// Titel     	: Basisstation fr Hausalarmanlage
//----------------------------------------------------------------------
// Schaltung 	: Pinbelegung siehe ports.h
// Beschreibung	: berwacht Reedkontakte und gibt Alarm.
//----------------------------------------------------------------------
// Prozessor 	: ATmega8
// Takt			: 8 MHz, intern
// Datenrate	: 9600 Baud
// Datum    	: 12.01.2018
// Version   	: s.u.
// Autor     	: P. Fuhsy
// Fuses	    : hfuse=0xC1, lfuse=0xE4, lockfuse=0xFF
/***********************************************************************/
//-----------------------------Includes---------------------------------
#include <avr/io.h>					//allg. Ports
#include <stdbool.h>				//fr Bool-Variablen
#include <avr/interrupt.h>			//fr Interrupts
#include <util/delay.h>				//fr Warteschleifen
#include <string.h>					//zur Stringbearbeitung
#include "uart.h"					//fr UART-bertragung
#include "ports.h"					//Zur Portdeklaration
#include "haa_bus.h"				//Bus der Alarmanlage
#include "esp8266.h"				//Wlan-Modul
//----------------------------------------------------------------------
/***********************************************************************/
//-----------------------Konstanten / Variablen-------------------------
#define SWH				5			//Software-Hauptversion
#define SWN				0			//Softwate-Nebenversion
#define T_SCHARF_S		30			//Totzeit bis Scharf geschaltet wird in sek.
#define T_ALARM_S		15			//Totzeit bis Voralarm zum Alarm wird in sek.
#define T_SUM_TOT_S		60			//Totzeit bis Summer nach dem Alarm ausgeschaltet wird in sek.
#define T_STATUS_S		10			//Intervall fr die Statusausgabe
#define ESP_TIMEOUT_S	30			//Timeout fr ESP-Anfragen

//Zustnde der Anlage
bool _kontakt_alarm		= false;	//Alarm der festverdrahteten Kontakte
bool _voralarm			= false;	//Voralarm wenn Eingangstr geffnet wird
bool _alarm				= false;	//allg. Alarm der Anlage
bool _anlage_start		= false;	//Bit wird gesetzt wenn die Anlage gerade startet
bool _anlage_scharf		= false;	//Bit wird gesetzt wenn die Anlage scharf ist
bool _sabotage_alarm	= false;	//Sabotagekontakte der Sirene
bool _stoerung			= false;	//Stoerungskontakt der Sirene
bool _reed[6]				   ;	//Reedkontakte im Array

//Zeitmessungen
volatile uint16_t _t_Led_Rot_ms;
volatile uint16_t _t_Led_Gruen_ms;
volatile uint16_t _t_Summer_ms;
volatile uint8_t _t_Sum_tot_s;
volatile uint8_t _t_Zustand_s;
volatile uint8_t _t_Voralarm_s;
volatile uint8_t _t_Status_s;
volatile uint8_t _ESP_Timeout_s;

volatile char _uart_string[50]			= "X";		//Stringbuffer fr UART
volatile static bool _uart_str_complete = false;	//String ist komplett
bool _ESP_Busy							= false;	//ESP ist beschftigt
bool _init_ESP							= false;	//ESP-Initialisierung abgeschlossen

//--------------------------Funktionsdefinitionen-----------------------
void ESP_Query (char *abfrage, char *antwort);
bool ESP_Response();
//------------------------------Funktionen------------------------------
void initPorts()			//Ports deklarieren
{
	//Output
	DDRD |= (1<<SUMMER);	//Summersausgang	
	DDRD |= (1<<SIRENE);	//Sirenenausgang	
	DDRD |= (1<<RELAIS);	//Relaisausgang	
	DDRB |= (1<<LED_RT_DATA);	//LED fr allg. Statuszustand
	DDRB |= (1<<LED_GN_OK);	//LED fr "Anlage nicht scharf"
	DDRC |= (1<<TEST_PORT);	//Testport zu Testzwecken	
	DDRD |= (1<<CS_EXIT);	//Externer Zugriff auf Codeschloss	

	//Input
	DDRC &=~(1<<REED_0);	//REED 0
	DDRC &=~(1<<REED_1);	//REED 1
	DDRC &=~(1<<REED_2);	//REED 2	
	DDRB &=~(1<<REED_3);	//REED 3
	DDRB &=~(1<<REED_4);	//REED 4
	DDRB &=~(1<<REED_5);	//REED 5
	DDRD &=~(1<<CS_REL1);	//Relais 1 vom Codeschloss	
	DDRD &=~(1<<SABOK);		//Sabotagekontakte	
	DDRB &=~(1<<STOERUNG);	//Strungssignal
	
	//interne Pull-Up-Widerstnde einschalten
	PORTC |= (1<<REED_0);	//REED 0		
	PORTC |= (1<<REED_1);	//REED 1	
	PORTC |= (1<<REED_2);	//REED 2	
	PORTB |= (1<<REED_3);	//REED 3
	PORTB |= (1<<REED_4);	//REED 4
	PORTB |= (1<<REED_5);	//REED 5
	PORTD |= (1<<CS_REL1);	//Relais 1 vom Codeschloss		
	PORTD |= (1<<SABOK);	//Sabotagekontakte	
	PORTB |= (1<<STOERUNG);	//Strungssignal			

	//Ports initiaisieren
	Summer_Off();			//Summerausgang
	Sirene_Off();			//Sirenausgang
	Relais_Off();			//Relais ausschalten		
	CS_Exit_Off();			//Exit-Ausgang ausschalten
	Led_Rot_Off();			//LED ausschalten
	Led_Gruen_Off();		//LED ausschalten
}
void initTimer() 			//Timer initialisierung
{
	//ISR_Freq = F_CPU : (Prescaler * ORC2) in Hz
	TCCR2 = (1<<CS22) | (1<<CS21) |(1<<WGM21);	//Vorteiler 256, CTC-Modus
	OCR2 = 31;								//Vergleichen: 
	TIMSK |= (1<<OCIE2);					//Compare Interrupt erlauben
}
void initESP8226()			//Modul fr den ersten Gebrauch initialisieren
{
	static int anfrage_i = 0;		//Laufvariable welche Anfrage gesendet wurde
		
	//Nur ausfhren wenn Timeout noch nicht erreicht
	if(_ESP_Timeout_s <= ESP_TIMEOUT_S)
	{
		//ESP ist verfgbar
		if(!_ESP_Busy)
		{
			//Welches Protkoll soll gesendet werden
			switch(anfrage_i)
			{
				//Anfragen senden
				case 0:
					ESP_Query(RESET, "ready");			//Modul reseten
				break;
				
// 				case 1:
// 					TCP_Laenge(BAS, SIR, REE, AKL, AKN, SWP, SWB, SWE);		//Zeichenlnge ermitteln
// 					//TCP_Laenge(0,0,0,0,0,18,80,0);		//Zeichenlnge ermitteln
// 				break;
							
				case 1:
					ESP_Query(WIFI_AUTO_CON, "OK");		//Wifi Autoconnect einschalten
				break;
										
				case 2:
					ESP_Query(WIFI_STA, "OK");			//WiFi-Mode STA					
				break;	
							
				case 3:
					ESP_Query(WIFI_JOIN, "OK");			//Join AP
				break;	
				
				case 4:
					ESP_Query(TCP_SINGLE_CON, "OK");	//TCP/UDP Connetion Single
				break;		
				
				case 5:
					ESP_Query(TCP_START, "OK");			//mit Empfnger verbinden
				break;					
						
				//alle Anfragen wurden gesendet
				default:
					anfrage_i = 0;			//Laufvariable zurcksetzten, falls init nochmal aufgerufen wird
					_init_ESP = true;		//Initialisierung abgeschlossen, Funktion wird nicht nochmal aufgerufen
					//putString("<INIT READY>");
				break;
			}
		}
	}
	
	//Timeout ist abgelaufen
	else
	{
		putString("<TIME ERROR>\r\n");
		_init_ESP = false;		//Merker zum Initialisieren zurcksetzten
		_ESP_Timeout_s = 0;		//Timer zurcksetzten
		_ESP_Busy = false;		//Zurcksetzten weil es sonst erst bei einer Antwort passiert
		anfrage_i = 0;			//Laufvariable zurcksetzten, damit init nochmal aufgerufen wird
	}

	//prfen ob erwartete Antwort vorhanden ist, erst dann kommt die nchste Abfrage
	if(ESP_Response())
	{
		anfrage_i++;		//variable fr nchste Anfrage
	}
}
void Zustand_Anlage()		//allg. Zustand der Alarmanlage abfragen
{
	//Relais 1 vom Codeschloss aktiv
	if (CSRel1_offen())						
	{
		_anlage_start = true;
		
		//Totzeit ist noch nicht abgelaufen.
		//Und prfen ob die Anlage scharf ist damit es nur einmals gesetzt wird.
		if (!_anlage_scharf && _t_Zustand_s <= T_SCHARF_S)
		{
			_anlage_scharf = false;
		}
		
		//Totzeit ist abgelaufen
		else
		{
			_anlage_scharf = true;
			_anlage_start = false;
			_t_Zustand_s = 0;
		}
	}
	
	//Relais 1 vom Codeschloss nicht aktiv
	else
	{
		_anlage_start = false;
		_anlage_scharf = false;
		_t_Zustand_s = 0;
	}
	
	//Sabotagekontakte		
	if(Sabotage())
	{
		_sabotage_alarm = true;
	}
	
	else
	{
		_sabotage_alarm = false;
	}
	
	//Strungsignal der Sirene
	if(Stoerung())
	{
		_stoerung = true;
	}
	
	else
	{
		_stoerung = false;
	}
	
	//Ein REEDkontakt ist geffnet
 	_reed[0] = Reed0_offen() ?  true : false;	//Auswertung, If-Kurzschreibweise
 	_reed[1] = Reed1_offen() ?  true : false;	//Auswertung, If-Kurzschreibweise
 	_reed[2] = Reed2_offen() ? true : false;	//Auswertung, If-Kurzschreibweise
	_reed[3] = Reed3_offen() ? true : false;	//Auswertung, If-Kurzschreibweise
 	_reed[4] = Reed4_offen() ? true : false;	//Auswertung, If-Kurzschreibweise
	_reed[5] = Reed5_offen() ? true : false;	//Auswertung, If-Kurzschreibweise	

	if ((Reed0_offen()) || (Reed1_offen()) || 
		(Reed2_offen()) || (Reed3_offen()) ||
		(Reed4_offen())	|| (Reed5_offen()))
	{
		_kontakt_alarm = true;
	}

	//Alle REEDkontakte sind geschlossen
	else
	{
		_kontakt_alarm = false;
	}
}
void Voralarm()				//Abfragen ob ein Voralarm vorhanden ist
{
	//Alarmanlage ist scharf gestellt
	if (_anlage_scharf)
	{	
		//REED 0 ist ausgelst
		if (Reed0_offen())
		{
			_voralarm = true;
		}
	}

	//Alarmanlage noch nicht scharf gestellt
	else
	{
		_voralarm = false;
		_t_Voralarm_s = 0;
	}

	//falls es noch kein Voralarm gibt muss der Zhler zurckgesetzt werden
	if (!_voralarm)
	{
		_t_Voralarm_s = 0;
	}

	//Bei Alarm wird der Zeitzhler und Voralarm zurckgesetzt
	if (_alarm)
	{
		_voralarm = false;
		_t_Voralarm_s = 0;
	}
}
void Alarm()				//Abfragen ob ein Alarm vorhanden ist
{
	//Alarmanlage ist scharf geschaltet
	if (_anlage_scharf)
	{	
		//REED 1, 2 oder 3 ist ausgelst (REED 0 ist die Tr)
		if ((Reed1_offen()) || (Reed2_offen()) || (Reed3_offen()) ||
			(Reed4_offen()) || (Reed5_offen()) || Sabotage())
		{
			_alarm = true;
		}

	}
	//Alarmanlage nicht scharf gestellt / wieder abgeschaltet
	else
	{
		_alarm = false;
	}
	
	if (_voralarm)
	{
		//Totzeit fr Voralarm ist abgelaufen
		if (_t_Voralarm_s > T_ALARM_S)
		{
			_alarm = true;
		}
	}
}
void Summer_Anstrg()		//Summer ansteuern
{
	static bool ausgeloest = false;	//Alarm schon ausgelst
	int16_t fall = 0;					//Fallunterscheidung
	int16_t t_an = 0;
	int16_t t_aus = 0;

	/*Fallunterscheidungen*/
	//Prio 4: allgemeine Strung
	if (_stoerung) {fall = 4;}
		
	//Prio 3: Kontaktalarm beim starten	
	if (_anlage_start && (Reed1_offen() || Reed2_offen() || Reed3_offen() || Reed4_offen() || Reed5_offen() || _sabotage_alarm)) {fall = 3;}

	//Prio 2: Voralarm
	if (_voralarm) {fall = 2;}	
			
	//Prio 1: Alarm ausgelst
	if (_alarm) {fall = 1;}		
		
	switch (fall)
	{
		//Alarm ausgelst
		case 1:
			//Alarm bereits ausgelst
			if (ausgeloest)
			{
				//falls die Totzeit abgelaufen ist
				if (_t_Sum_tot_s >= T_SUM_TOT_S)
				{
					Summer_Off();
					_t_Sum_tot_s = 0;
				}
			}

			//Alarm noch nicht ausgelst
			else
			{
				ausgeloest = true;
				Summer_On();
			}
		break;
		
		//Kontaktalarm beim Starten
		case 2:
			//Ein-Auszeiten des Summers
			t_an = 100;
			t_aus = 900;
			
			//Summer an
			if(_t_Summer_ms <= t_an)
			{
				Summer_On();
			}
		
			else
			{
				//Summer aus
				if (_t_Summer_ms <= t_an + t_aus)
				{
					Summer_Off();
				}
			
				//Timer zurcksetzten
				else
				{
					_t_Summer_ms = 0;
				}
			}
		break;
		
		//Voralarm ausgelst
		case 3:
			//Ein-Auszeiten des Summers
			t_an = 100;
			t_aus = 900;
			
			//Summer an
			if(_t_Summer_ms <= t_an)
			{
				Summer_On();
			}
						
			else
			{
				//Summer aus
				if (_t_Summer_ms <= t_an + t_aus)
				{
					Summer_Off();
				}
							
				//Timer zurcksetzten
				else
				{
					_t_Summer_ms = 0;
				}
			}
		break;
		
		//Strung ausgelst
		case 4:
		//Ein-Auszeiten des Summers
		t_an = 500;
		t_aus = 29500;
		
		//Summer an
		if(_t_Summer_ms <= t_an)
		{
			Summer_On();
		}
		
		else
		{
			//Summer aus
			if (_t_Summer_ms <= t_an + t_aus)
			{
				Summer_Off();
			}
			
			//Timer zurcksetzten
			else
			{
				_t_Summer_ms = 0;
			}
		}
		break;		
				
		default:
			Summer_Off();
			ausgeloest = false;
			_t_Summer_ms = 0;
			_t_Sum_tot_s = 0;
		break;
	}
}
void Sirene_Anstrg()		//Sirene ansteuern
{
	//Alarm wurde ausgelst
	if (_alarm)
	{
		Sirene_On();
	}

	//Alarm nicht ausgelst
	else
	{
		Sirene_Off();
	}
}
void Led_Rot_Anstrg()		//LED rot fr allg. Betriebszustnde setzten
{
	static int16_t t_an = 0;
	static int16_t t_aus = 0;
	
	//-------------Blinkzeiten festlegen-------------------
	//Blinkzeiten setzten, Reihenfolge beachten
	//Anlage startet gerade
	if (_anlage_start)
	{
		t_an = 500;	//in Milisekunden
		t_aus = 500;
	}

	//Anlage ist scharf
	if (_anlage_scharf)
	{
		t_an = 100;	//in Milisekunden
		t_aus = 2900;
	}

	//Voralarm ist vorhanden
	if (_voralarm)
	{
		t_an = 250;	//in Milisekunden
		t_aus = 250;
	}
	//-----------------------------------------------------

	//------------------LED-Blinker------------------------
	//Wenn einer davon auf True ist, fngt die LED an zu blinken
	if (_voralarm || _anlage_start || _anlage_scharf)
	{

		//Dauerlicht fr Alarm
		if (_alarm)
		{
			Led_Rot_On();
			_t_Led_Rot_ms = 0;	//globale Zeitvariable zurcksetzten
		}

		//..ansonsten Blinken ansetzten
		//LED an
		else if (_t_Led_Rot_ms <= t_an)
		{
			Led_Rot_On();
		}
		//LED aus
		else if (_t_Led_Rot_ms <= t_an + t_aus)
		{
			Led_Rot_Off();
		}
		else
		{
			_t_Led_Rot_ms = 0;
		}
		
	}

	//Anlage abgeschaltet
	else
	{
		Led_Rot_Off();
		_t_Led_Rot_ms = 0;
	}
	//-----------------------------------------------------
}
void Led_Gruen_Anstrg()		//LED grn fr "Alles OK" setzten
{
	static int16_t t_an = 500;
	static int16_t t_aus = 500;
	
	//LED einschalten wenn die Anlage nicht scharf ist
	if (!_anlage_scharf)
	{
		
		//Ein Reed- oder Sabotagekontakt ist geffnet
		if (_kontakt_alarm || _sabotage_alarm)
		{
			//LED blinken: AN
			if (_t_Led_Gruen_ms <= t_an)
			{
				Led_Gruen_On();
			}
			//LED blinken_ AUS
			else if (_t_Led_Gruen_ms <= t_an + t_aus)
			{	
				Led_Gruen_Off();
			}
			else
			{
				_t_Led_Gruen_ms = 0;
			}	
		}

		//Alle Kontakte sind geschlossen
		else
		{
			Led_Gruen_On();
			_t_Led_Gruen_ms = 0;
		}
	}

	//bei scharfer Alarmanlage LED ausschalten
	else
	{
		Led_Gruen_Off();
		_t_Led_Gruen_ms = 0;
	}
}
void Power_On()				//Power On Prozedur aller Ausgnge
{
	//Ausgnge ansteuern
	Led_Rot_On();
	Led_Gruen_On();	
	Summer_On();
	_delay_ms(2000);
	
	//Ausgnge zurcksetzten
	Led_Rot_Off();
	Led_Gruen_Off();
	Summer_Off();
	_delay_ms(1000);		
}
void Haa_Bus_schreiben()	//Haa-bus beschreiben damit zusammengefasst und gesendet wird
{
	BAS_byte_schreiben(_sabotage_alarm, _anlage_scharf, _anlage_start, _alarm, _voralarm, _kontakt_alarm);
	SIR_byte_schreiben(_stoerung);	//Statusmeldungen der der Sirene
	REE_byte_schreiben(_reed);		//als Array bertragen (Zeiger des ersten Arraywertes)
	//AKL_byte_schreiben(Akkuspannung);	//Resereve fr Akkuspannung
	//AKN_byte_schreiben(Akkuspannung);	//Resereve fr Akkuspannung
	SWP_byte_schreiben(SWH, SWN);		//SW-Version

}
void Protokoll_senden()		//Ausgabe des Protokolls
{		
	static int anfrage_i = 0;		//Laufvariable welche Anfrage gesendet wurde
	
	//Ausgabe erst nach angegebener Zeit		
	if (_t_Status_s >= T_STATUS_S)		
	{
		//Nur ausfhren wenn Timeout noch nicht erreicht
		if(_ESP_Timeout_s <= ESP_TIMEOUT_S)
		{		
			if(!_ESP_Busy)	//ESP steht zur Verfgung und kann angefragt werden
			{			
					//Welches Protkoll soll gesendet werden
					switch(anfrage_i)
					{
						//Anfragen senden
						case 0:
							TCP_Laenge(BAS, SIR, REE, AKL, AKN, SWH, SWN);		//Zeichenlnge ermitteln
							//TCP_Laenge(8,0,0,0,0,17,80,0);		//Zeichenlnge ermitteln
							ESP_Query(_cipsend, "OK");
 							//putString("AT+CIPSEND=101\r\n");									
 							//_ESP_Timeout_s = 0;		//Timeout-Zeit zurcksetzten
							//_ESP_Busy = true;		//Flag ESP ist beschftigt
						break;

						case 1:
							putString(_get);		//TODO als Funktion schreiben
							putString(_host);					
							//putString("GET /esp8266/haawrite.php?b0=8&b1=0&b2=0&b3=0&b4=0&b5=17&b6=80&b7=0 HTTP/1.1\r\n"); //\r\n werden als 2 Zeichen gezhlt. Also kompletten String immer -6 rechnen
							//putString("Host: 192.168.1.100\r\n\r\n");
							//putString("Connection: keep-alive\r\n");
							_ESP_Timeout_s = 0;		//Timeout-Zeit zurcksetzten, sonst wre das mit der ESP-Query enthalten
							_ESP_Busy = true;		//Flag ESP ist beschftigt, sonst wre das mit der ESP-Query enthalten						
						break;							
				
						//alle Anfragen wurden gesendet
						default:
							_t_Status_s = 0;		//Timer zurcksetzten, damit in angegebener Zeit wieder gesenet wird
							anfrage_i = 0;			//Laufvariable zurcksetzten, damit die Funktion nochmal aufgerufen werden kann
						break;				
					}			
 			}
		 
			else
			{
 				//prfen ob erwartete Antwort vorhanden ist
 				if(ESP_Response())
 				{
	 				anfrage_i++;			//variable fr nchste Anfrage
 				}	
			}
		}
		//Timeout ist abgelaufen
		else
		{
			//TODO Funktion draus machen weil es mehrmals verwendet wird
			//TODO Errors zhlen und in XML schreiben
			putString("<TIME ERROR>\r\n");
			_init_ESP = false;		//Merker zum Initialisieren zurcksetzten
			_ESP_Timeout_s = 0;		//Timer zurcksetzten
			_ESP_Busy = false;		//Zurcksetzten weil es sonst erst bei einer Antwort passiert
			anfrage_i = 0;			//Laufvariable zurcksetzten, damit init nochmal aufgerufen wird
		}			
	}
}
void ESP_Query (char *anfrage, char *antwort)
{
	strcpy(_gesuchte_antw, antwort);	//erwartete Antwort abspeichern
	putString(anfrage);					//Anfrage senden
	_ESP_Timeout_s = 0;					//Timeout-Zeit zurcksetzten
	_ESP_Busy = true;					//Flag ESP ist beschftigt
}
bool ESP_Response()
{
	bool rueckgabe = false;

	//Abfrage ob Antwort vorhanden ist
	if(_uart_str_complete)
	{
		//prfen ob die gesuchte Antwort vorhanden ist
		if(strstr(_uart_string, _gesuchte_antw))	//nach einzelnen Wrtern suchen (ohne Leerzeichen)
		{
			_ESP_Busy = false;		//ESP wieder freigeben				
			rueckgabe = true;		//Rckgabewert positiv
		}
				
		//Unabhngig der Antwort, alles zurcksetzten
		memset(_uart_string, '\0', sizeof(_uart_string));	//Inhalte vom Array lschen
		_uart_str_complete = false;							//String komplett zurcksetzten		
	}

	return rueckgabe;
}
void Schleifenzaehler()		//Zu Testzwecken, Anzahl der Wiederholungen in einer Scheife zhlen und anzeigen
{
// 	static int x = 0;
// 	static char wiederholung[5];
// 	
// 	x++;
// 	sprintf(wiederholung, "%i", x);			//Wandlung von Int in Char Array
// 	putString(wiederholung);
}

int main(void)				/***Hauptprogramm***/
{
	initPorts();			//Ports deklarieren
	initUART();				//UART initialisieren
	initTimer();			//Timer initialisieren
	Power_On();				//Power On Prozedur aller Ausgnge
	sei();					//alle Interrups einschalten 

	while(true) //Hauptschleife
	{				
 		Zustand_Anlage();	//Anlagenzustand abfragen
 		Voralarm();			//Abfragen ob ein Voralarm vorhanden ist
 		Alarm();			//Abfragen ob ein Alarm vorhanden ist
 		Summer_Anstrg();	//Ausgang des Summers setzten
		Sirene_Anstrg();	//Ausgang der Sirene setzten
  		Led_Gruen_Anstrg();	//Ausgang der grnen LED setzten
 		Led_Rot_Anstrg();	//Ausgang der roten LED setzten
		Haa_Bus_schreiben();//Bus fr bertragung und Anzeige beschreiben
		
		if (!_init_ESP)		//nur ausfhren, wenn noch nicht abgeschlossen
		{
			initESP8226();
		}
		
		else
		{
			Protokoll_senden();	//Bus-Protokoll senden
		}		
	}
}
ISR (TIMER2_COMP_vect)		//Time-Compare-Interrupt, wird jede ms ausgelst
{
	static uint16_t millisek = 0;
	
	//Millisekungen zhlen	
	_t_Led_Rot_ms ++;
	_t_Led_Gruen_ms ++;
	_t_Summer_ms ++;

	millisek++;
		
	//Sekunden zhlen
	if (millisek == 999)
	{
		//Funktionszhler
		_t_Sum_tot_s++;
		_t_Zustand_s ++;
		_t_Status_s ++;
		_t_Voralarm_s ++;
		_ESP_Timeout_s ++;	
		
		millisek = 0;	//Zhler zurcksetzten
	}	
}
ISR (USART_RXC_vect)		//Interrupt wird ausgelst sobald neue Daten im USART-Empfangspuffer liegen
{
	volatile static uint8_t count = 0;		//Char-Zhler
	
	//String ist noch nicht komplett
	if (!_uart_str_complete)
	{
		//kein Abschlusszeichen vorhanden
		if(UDR != '\n')		
		{
			_uart_string[count] = UDR;				//in Array schreiben
			count++;								//Array-Zhler inkrementieren
		}
		else
		{
			_uart_string[count]='\0';				//String abschliessen
			count = 0;								//Zhler zurcksetzten
			_uart_str_complete = true;				//String komplett
		}
		
		//Arraylnge erreicht
		if (count >= sizeof(_uart_string) - 1)		
		{
			count = 0;								//Zhler zurcksetzten
		}
	}		
	
}